@@ -149,7 +149,10 @@ def _report_error(message, test_name, mode, start_test, error=True):
     else:
         fishy = "WARNING"
     print "\n%s%s%s: %s\n" % (BOLD_ON, fishy, BOLD_OFF, message)
-    failed_tests.append(test)
+    if mode == "FAIL":
+        failed_tests.append(test)
+    else:
+        skipped_tests.append(test)
 
 # Helper method to manage exception handling
 def _exec_and_report(procedure, default_message, test_name, action,
@@ -243,11 +246,15 @@ def find_tests(path):
                     pass
                 else:
                     if opt.wildcard:
-                        if opt.wildcard == fname[0:len(opt.wildcard)]:
-                            test_files.append(test_ref)
+                        for wild in opt.wildcard:
+                            if wild == fname[0:len(wild)]:
+                                test_files.append(test_ref)
+                                break
                     elif opt.skip_tests:
-                        if opt.skip_tests != fname[0:len(opt.skip_tests)]:
-                            test_files.append(test_ref)
+                        for skip in opt.skip_tests:
+                            if skip != fname[0:len(skip)]:
+                                test_files.append(test_ref)
+                                break
                     # Add test if no wildcard and suite (dir) is included
                     else:
                         test_files.append(test_ref)
@@ -268,9 +275,10 @@ parser.add_option("--server", action="append", dest="servers",
                   "- list option multiple times for multiple servers to use")
 
 # Add test wildcard option
-parser.add_option("--do-tests", action="store", dest="wildcard",
+parser.add_option("--do-tests", action="append", dest="wildcard",
                   type = "string", help="execute all tests that begin " \
-                         "with this string")
+                         "with this string. List option multiple times "
+                         "to add multiple wildcards.")
 
 # Add suite list option
 parser.add_option("--suite", action="append", dest="suites",
@@ -279,19 +287,26 @@ parser.add_option("--suite", action="append", dest="suites",
 
 # Add skip-test list option
 parser.add_option("--skip-test", action="append", dest="skip_test",
-                  type = "string", help="exclude a test - list "
+                  type = "string", help="exclude a specific test - list "
                   "option multiple times for multiple tests")
 
 # Add skip-test list option
-parser.add_option("--skip-tests", action="store", dest="skip_tests",
+parser.add_option("--skip-tests", action="append", dest="skip_tests",
                   type = "string", help="exclude tests that begin with "
-                  "this string")
+                  "this string. List option multiple times to add "
+                  "multiple skips.")
 
 # Add start-test list option
 parser.add_option("--start-test", action="store", dest="start_test",
                   type = "string", help="start executing tests that begin "
                   "with this string", default=None)
 
+# Add start-test list option
+parser.add_option("--stop-test", action="store", dest="stop_test",
+                  type = "string", help="stop executing tests at first "
+                  "occurrence of test that begins with this string",
+                  default=None)
+
 # Add skip-long tests option
 parser.add_option("--skip-long", action="store_true", dest="skip_long",
                   default=False, help="exclude tests that require "
@@ -339,6 +354,10 @@ parser.add_option("-f", "--force", action="store_true", dest="force",
 # Add verbosity mode
 add_verbosity(parser, False)
 
+# Skip cleanup mode
+parser.add_option("--skip-cleanup", action="store_true", dest="skip_cleanup",
+                  help="Do not shutdown spawned servers - debug only mode.")
+
 # Now we process the rest of the arguments.
 opt, args = parser.parse_args()
 
@@ -390,7 +409,8 @@ if opt.skip_suites:
 
 # Is there a --do-test?
 if opt.wildcard:
-    print "  Test wildcard       = '%s%%'" % opt.wildcard
+    for wild in opt.wildcard:
+        print "  Test wildcard       = '%s%%'" % wild
 
 # Check to see if we're skipping tests
 if opt.skip_test:
@@ -400,7 +420,8 @@ if opt.skip_test:
     print
 
 if opt.skip_tests:
-    print "  Skip wildcard       = '%s%%'" % opt.skip_tests
+    for skip in opt.skip_tests:
+        print "  Skip wildcard       = '%s%%'" % skip
 
 if opt.start_test:
     print "  Start test sequence = '%s%%'" % opt.start_test
@@ -408,6 +429,9 @@ if opt.start_test:
 else:
     start_sequence = False
 
+if opt.stop_test:
+    print "  Stop test           = '%s%%'" % opt.stop_test
+
 server_list = Server_list([], opt.start_port, opt.utildir, verbose_mode)
 basedir = None
 
@@ -477,6 +501,7 @@ if len(processes) > 0:
 
 test_files = []
 failed_tests = []
+skipped_tests = []
 
 test_files.extend(find_tests(SUITE_PATH))                    
 test_files.extend(find_tests(opt.testdir))
@@ -499,6 +524,17 @@ if start_sequence:
               "suite(s)" % opt.start_test
         start_sequence = False
 
+# Check for validity of --stop-test
+if opt.stop_test:
+    found = False
+    for test_tuple in test_files:
+        if opt.stop_test == test_tuple[2][0:len(opt.stop_test)]:
+            found = True
+    if not found:
+        print "\nWARNING: --stop-test=%s%% was not found. Running full " \
+              "suite(s)" % opt.stop_test
+        opt.stop_test = None
+
 # Get list of disabled tests
 disable_list = _read_disabled_tests()
 
@@ -517,6 +553,7 @@ if os.name == "posix":
 # Run the tests selected
 num_tests_run = 0
 last_test = None
+stop_testing = False
 for test_tuple in test_files:
 
     # Skip tests for start-test sequence
@@ -526,6 +563,12 @@ for test_tuple in test_files:
         else:
             continue
 
+    # Skip tests for stop-test sequence
+    if opt.stop_test and not stop_testing:
+        stop_testing = (opt.stop_test == test_tuple[2][0:len(opt.stop_test)])
+    if stop_testing:
+        continue
+
     # Get test parts - directory not used
     test = test_tuple[2]
     
@@ -675,27 +718,35 @@ print "-" * opt.width
 print datetime.datetime.now().strftime("Testing completed: "
                                        "%A %d %B %Y %H:%M:%S\n")
 num_fail = len(failed_tests)
+num_skip = len(skipped_tests)
 num_tests = len(test_files)
-if num_fail == 0:
+if num_fail == 0 and num_skip == 0:
     if num_tests > 0:
         print "All %d tests passed." % (num_tests)
 else:
-    print "%d of %d tests completed." % \
+    print "%d of %d tests completed.\n" % \
           (num_tests_run, num_tests)
-    print "The following tests failed or were skipped:",
-    for test in failed_tests:
-        print test,
-    print "\n"
+    if num_skip:
+        print "The following tests were skipped:\n",
+        for test in skipped_tests:
+            print test,
+        print "\n"
+    if num_fail:
+        print "The following tests failed:\n",
+        for test in failed_tests:
+            print test,
+        print "\n"
 
 # Shutdown connections and spawned servers
-if server_list.num_spawned_servers():
-    print "\nShutting down spawned servers "
-    server_list.shutdown_spawned_servers()
-
-if (server_list.cleanup_list) > 0:
-    sys.stdout.write("\nDeleting temporary files...")
-    server_list.remove_files()
-    print "success."
+if not opt.skip_cleanup:
+    if server_list.num_spawned_servers():
+        print "\nShutting down spawned servers "
+        server_list.shutdown_spawned_servers()
+    
+    if (server_list.cleanup_list) > 0:
+        sys.stdout.write("\nDeleting temporary files...")
+        server_list.remove_files()
+        print "success."
 
 del server_list
 
