@@ -13,7 +13,8 @@ class test(mutlib.System_test):
     """
 
     def check_prerequisites(self):
-        self.check_gtid_unsafe()
+        if self.servers.get_server(0).check_version_compat(5, 6, 5):
+            raise MUTLibError("Test requires server version prior to 5.6.5")
         return self.check_num_servers(1)
 
     def setup(self):
@@ -43,8 +44,8 @@ class test(mutlib.System_test):
             if self.debug:
                 print comment
             self.drop_db(self.server1, "util_db_clone")
-            cmd = "mysqldbcopy.py %s %s " % (from_conn, to_conn) + \
-                  " util_test:util_db_clone --force --locking=%s" % locktype
+            cmd = "mysqldbcopy.py --skip-gtid %s %s util_test:util_db_clone " \
+                  " --force --locking=%s" % (from_conn, to_conn, locktype)
             try:
                 res = self.exec_util(cmd, self.res_fname)
                 self.results.append(res)
