@@ -186,7 +186,7 @@ class ProcessGrep(object):
           format           format for display
                            default = GRID
         """
-        from mysql.utilities.exception import EmptyResultError
+        from mysql.utilities.exception import EmptyResultError, FormatError
         from ..common.options import parse_connection
         from ..common.format import print_list
 
@@ -217,6 +217,7 @@ class ProcessGrep(object):
         
         # If output is None, nothing is printed
         if len(entries) > 0 and output:
+            entries.sort(key = lambda fifth:fifth[5]) 
             print_list(output, format, headers, entries)
         elif PRINT_PROCESS in self.__actions:
             raise EmptyResultError("No matches found")
