@@ -24,7 +24,7 @@ simple master-to-slaves topology.
 import logging
 import os
 import sys
-from mysql.utilities.exception import UtilError, UtilRplError
+from mysql.utilities.exception import UtilRplError
 
 _VALID_COMMANDS_TEXT = """
 Available Commands:
@@ -39,6 +39,9 @@ Available Commands:
   stop        - stop all slaves
   switchover  - perform slave promotion
   
+  Note: elect, failover, gtid, and health require --master and either
+        --slaves or --discover-slave-login
+  
 """
 
 _VALID_COMMANDS = ["elect", "failover", "gtid", "health", "reset", "start",
@@ -55,6 +58,11 @@ _FAILOVER_ERRNO = 911
 _DATE_FORMAT = '%Y-%m-%d %H:%M:%S %p'
 _DATE_LEN = 22
 
+_HOST_IP_WARNING = "You may be mixing host names and IP " + \
+                   "addresses. This may result in negative status " + \
+                   "reporting if your DNS services do not support " + \
+                   "reverse name lookup."
+
 def get_valid_rpl_command_text():
     """Provide list of valid command descriptions to caller.
     """
@@ -149,6 +157,16 @@ class RplCommands(object):
         self.quiet = self.options.get("quiet", False)
         self.logging = self.options.get("logging", False)
         self.candidates = self.options.get("candidates", None)
+
+        # Replace all local host IP addresses (i.e. 127.0.0.1) by localhost
+        for candidate in self.candidates:
+            if candidate['host'] == '127.0.0.1':
+                candidate['host'] = 'localhost'
+        for slave in slave_vals:
+            if slave['host'] == '127.0.0.1':
+                slave['host'] = 'localhost'
+        
+        self.rpl_user = self.options.get("rpl_user", None)
         self.topology = Topology(master_vals, slave_vals, self.options,
                                  skip_conn_err)
         
@@ -259,6 +277,24 @@ class RplCommands(object):
             print_list(sys.stdout, format, _GTID_COLS, owned)
 
 
+    def _check_host_references(self):
+        """Check to see if using all host or all IP addresses
+        
+        Returns bool - True = all references are consistent
+        """
+        from mysql.utilities.common.options import hostname_is_ip
+        
+        uses_ip = hostname_is_ip(self.topology.master.host)
+        for slave_dict in self.topology.slaves:
+            slave = slave_dict['instance']
+            if slave is not None:
+                host, port = slave.get_master_host_port()
+                if uses_ip != hostname_is_ip(slave.host) or \
+                   uses_ip != hostname_is_ip(host):
+                    return False
+        return True
+                
+
     def _switchover(self):
         """Perform switchover from master to candidate slave
         
@@ -267,9 +303,15 @@ class RplCommands(object):
         
         Returns bool - True = no errors, False = errors reported.
         """
-        from mysql.utilities.exception import FormatError
-        from mysql.utilities.common.options import parse_connection
-        
+        # Check for --master-info-repository=TABLE if rpl_user is None
+        if not self._check_master_info_type():
+            return False
+
+        # Check for mixing IP and hostnames
+        if not self._check_host_references():
+            print "# WARNING: %s" % _HOST_IP_WARNING
+            self._report(_HOST_IP_WARNING, logging.WARN, False)
+
         # Check prerequisites - need valid candidate
         candidate = self.options.get("new_master", None)
         if candidate is None:
@@ -299,6 +341,12 @@ class RplCommands(object):
             self._report("# WARNING: slave election requires GTID_MODE=ON "
                          "for all servers.", logging.WARN)
             return
+
+        # Check for mixing IP and hostnames
+        if not self._check_host_references():
+            print "# WARNING: %s" % _HOST_IP_WARNING
+            self._report(_HOST_IP_WARNING, logging.WARN, False)
+
         candidates = self.options.get("candidates", None)
         if candidates is None or len(candidates) == 0:
             self._report("# Electing candidate slave from known slaves.")
@@ -331,12 +379,38 @@ class RplCommands(object):
             self._report("# WARNING: slave election requires GTID_MODE=ON "
                          "for all servers.", logging.WARN)
             return
+        
+        # Check for --master-info-repository=TABLE if rpl_user is None
+        if not self._check_master_info_type():
+            return False
+
         self._report("# Performing failover.")
         if not self.topology.failover(self.candidates, strict):
             self._report("# Errors found.", logging.ERROR)
             return False
         return True
+
+
+    def _check_master_info_type(self, halt=True):
+        """Check for master information set to TABLE if rpl_user not provided
         
+        halt[in]       if True, raise error on failure. Default is True
+        
+        Returns bool - True if rpl_user is specified or False if rpl_user not
+                       specified and at least one slave does not have
+                       --master-info-repository=TABLE.
+        """
+        error = "You must specify either the --rpl-user or set all slaves " + \
+                "to use --master-info-repository=TABLE."
+        # Check for --master-info-repository=TABLE if rpl_user is None
+        if self.rpl_user is None:
+            if not self.topology.check_master_info_type("TABLE"):
+                if halt:
+                    raise UtilRplError(error)
+                self._report(error, logging.ERROR)
+                return False
+        return True
+
 
     def execute_command(self, command):
         """Execute a replication admin command
@@ -400,6 +474,69 @@ class RplCommands(object):
     def auto_failover(self, interval):
         """Automatic failover
         
+        Wrapper class for running automatic failover. See
+        run_automatic_failover for details on implementation.
+        
+        This method ensures the registration/deregistration occurs
+        regardless of exception or errors.
+
+        interval[in]   time in seconds to wait to check status of servers
+        
+        Returns bool - True = success, raises exception on error
+        """
+        import time
+        from mysql.utilities.command.failover_console import FailoverConsole
+
+        failover_mode = self.options.get("failover_mode", "auto")
+        force = self.options.get("force", False)
+
+        # Initialize a console
+        console = FailoverConsole(self.topology.master,
+                                  self.topology.get_health,
+                                  self.topology.get_gtid_data,
+                                  self.topology.get_server_uuids,
+                                  self.options)
+        
+        # Register instance
+        self._report("Registering instance on master.", logging.INFO, False)
+        old_mode = failover_mode
+        failover_mode = console.register_instance(force)
+        if failover_mode != old_mode:
+            self._report("Multiple instances of failover console found for "
+                         "master %s:%s." % (self.topology.master.host,
+                                            self.topology.master.port),
+                         logging.WARN)
+            print "If this is an error, restart the console with --force. "
+            print "Failover mode changed to 'FAIL' for this instance. "
+            print "Console will start in 10 seconds.",
+            sys.stdout.flush()
+            for i in range(0, 9):
+                time.sleep(1)
+                sys.stdout.write('.')
+                sys.stdout.flush()
+            print "starting Console."
+            time.sleep(1)
+        
+        try:
+            res = self.run_auto_failover(console, interval);
+        except:
+            raise
+        finally:
+            try:
+                # Unregister instance
+                self._report("Unregistering instance on master.", logging.INFO,
+                             False)
+                console.register_instance(False, False)
+                self._report("Failover console stopped.", logging.INFO, False)
+            except:
+                pass
+
+        return res
+        
+
+    def run_auto_failover(self, console, interval):
+        """Run automatic failover
+        
         This method implements the automatic failover facility. It uses the
         FailoverConsole class from the failover_console.py to implement all
         user interface commands and uses the existing failover() method of
@@ -411,20 +548,19 @@ class RplCommands(object):
         2) failover to list of candidates only
         3) fail
             
+        console[in]    instance of the failover console class
         interval[in]   time in seconds to wait to check status of servers
         
         Returns bool - True = success, raises exception on error
         """
         import time
-        from mysql.utilities.command.failover_console import FailoverConsole
         from mysql.utilities.common.tools import ping_host
         from mysql.utilities.common.tools import execute_script
         
         failover_mode = self.options.get("failover_mode", "auto")
         pingtime = self.options.get("pingtime", 3)
-        timeout = self.options.get("timeout", 3)
+        timeout = int(self.options.get("timeout", 3))
         exec_fail = self.options.get("exec_fail", None)
-        force = self.options.get("force", False)
         post_fail = self.options.get("post_fail", None)
                 
         # Only works for GTID_MODE=ON
@@ -441,39 +577,35 @@ class RplCommands(object):
             msg = "User %s on %s does not have sufficient privileges to " + \
                   "execute the %s command."
             for error in errors:
-                self._report(msg % (error[0], error[1], command),
+                self._report(msg % (error[0], error[1], 'failover'),
                                     logging.CRITICAL)
             raise UtilRplError("Not enough privileges to execute command.")
+            
+        # Require --master-info-repository=TABLE for all slaves
+        if not self.topology.check_master_info_type("TABLE"):
+            msg = "Failover requires --master-info-repository=TABLE for " + \
+                  "all slaves."
+            self._report(msg, logging.ERROR, False)
+            raise UtilRplError(msg)
+        
+        # Check for mixing IP and hostnames
+        if not self._check_host_references():
+            print "# WARNING: %s" % _HOST_IP_WARNING
+            self._report(_HOST_IP_WARNING, logging.WARN, False)
+            print "#\n# Failover console will start in 10 seconds."
+            time.sleep(10)
 
         # Test failover script. If it doesn't exist, fail.
         no_exec_fail_msg = "Failover check script cannot be found. Please " + \
                            "check the path and filename for accuracy and " + \
                            "restart the failover console."
-        if exec_fail is not None and not os.path.exists(fail_check):
+        if exec_fail is not None and not os.path.exists(exec_fail):
             self._report(no_exec_fail_msg, logging.CRITICAL, False)
             raise UtilRplError(no_exec_fail_msg)
                
-        # Initialize a console
-        console = FailoverConsole(self.topology.master, self.topology.get_health,
-                                  self.topology.get_gtid_data,
-                                  self.topology.get_server_uuids,
-                                  self.options)
-        
-        # Register instance
-        self._report("Registering instance on master.", logging.INFO, False)
-        old_mode = failover_mode
-        failover_mode = console.register_instance(force)
-        if failover_mode != old_mode:
-            self._report("Multiple instances of failover console found for "
-                         "master %s:%s." % (self.topology.master.host,
-                                            self.topology.master.port),
-                         logging.WARN)
-            print "Failover mode changed to 'FAIL'. Console will start in 5 seconds."
-            time.sleep(5)
-        
         self._report("Failover console started.", logging.INFO, False)
         self._report("Failover mode = %s." % failover_mode, logging.INFO, False)
-        
+       
         # Main loop - loop and fire on interval.
         done = False
         first_pass = True
@@ -489,7 +621,7 @@ class RplCommands(object):
                 else:
                     self._report("# Spawning external script for failover "
                                  "checking.")
-                    res = execute_script(script)
+                    res = execute_script(exec_fail)
                     if res == 0:
                         self._report("# Failover check script completed Ok. "
                                      "Failover averted.")
@@ -561,8 +693,8 @@ class RplCommands(object):
                 self.topology.run_script(post_fail, False)
 
             # discover slaves if option was specified at startup
-            elif self.options.get("discover", None) is not None \
-                and not first_pass:
+            elif self.options.get("discover", None) is not None and \
+                (not first_pass or self.options.get("rediscover", False)):
                 # Force refresh of health list if new slaves found
                 if self.topology.discover_slaves():
                     console.list_data = None
@@ -574,9 +706,4 @@ class RplCommands(object):
                 done = True        # User has quit
             first_pass = False
 
-        # Unregister instance
-        self._report("Unregistering instance on master.", logging.INFO, False)
-        console.register_instance(False, False)
-        self._report("Failover console stopped.", logging.INFO, False)
-
         return True
