@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2011 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2012 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,12 +23,13 @@ server operations used in multiple utilities.
 
 import os
 import re
-import sys
 import mysql.connector
-from mysql.utilities.exception import UtilError, UtilDBError
+from mysql.utilities.exception import UtilError, UtilDBError, UtilRplError
 from mysql.utilities.common.options import parse_connection
 
 _FOREIGN_KEY_SET = "SET foreign_key_checks = %s"
+_GTID_ERROR = ("The server %s:%s does not comply to the latest GTID " 
+               "feature support. Errors:")
 
 def get_connection_dictionary(conn_info):
     """Get the connection dictionary.
@@ -105,6 +106,7 @@ def get_local_servers(all=False, start=3306, end=3333, datadir_prefix=None):
     import string
     import subprocess
     import tempfile
+    from mysql.utilities.common.tools import execute_script
 
     processes = []
     if os.name == "posix":
@@ -146,10 +148,7 @@ def get_local_servers(all=False, start=3306, end=3333, datadir_prefix=None):
                     processes.append((proc_id, datadir_arg[10:]))
     elif os.name == "nt":
         f_out = open("portlist", 'w+')
-        proc = subprocess.Popen("netstat -anop tcp", shell=True,
-                                stdout=f_out, stderr=f_out)
-        res = proc.wait()
-        f_out.close()
+        res = execute_script("netstat -anop tcp", "portlist")
         f_out = open("portlist", 'r')
         for line in f_out.readlines():
             proginfo = string.split(line)
@@ -171,8 +170,11 @@ def get_local_servers(all=False, start=3306, end=3333, datadir_prefix=None):
     return processes
 
 
-def _connect_server(name, values, quiet):        
-    """Connect to a server
+def get_server(name, values, quiet):        
+    """Connect to a server and return Server instance
+    
+    If the name is 'master' or 'slave', the connection will be made via the
+    Master or Slave class else a normal Server class shall be used.
     
     name[in]           name of the server
     values[in]         dictionary of connection values
@@ -180,6 +182,9 @@ def _connect_server(name, values, quiet):
     
     Returns Server class instance
     """
+    from mysql.utilities.common.replication import Master
+    from mysql.utilities.common.replication import Slave
+    
     server_conn = None
 
     # Try to connect to the MySQL database server.
@@ -190,11 +195,17 @@ def _connect_server(name, values, quiet):
         'conn_info' : values,
         'role'      : name,
     }
-    server_conn = Server(server_options)
+    if name.lower() == 'master':
+        server_conn = Master(server_options)
+    elif name.lower() == 'slave':
+        server_conn = Slave(server_options)
+    else:
+        server_conn = Server(server_options)
     server_conn.connect()
 
     return server_conn
- 
+
+
 def _require_version(server, version):
     """Check version of server
     
@@ -210,6 +221,42 @@ def _require_version(server, version):
     return True
         
 
+def get_server_state(server, host, pingtime=3, verbose=False):
+    """Return the state of the server.
+    
+    This method returns one of the following states based on the
+    criteria shown.
+    
+      UP   - server is connected
+      WARN - server is not connected but can be pinged
+      DOWN - server cannot be pinged nor is connected
+
+    server[in]     Server class instance
+    host[in]       host name to ping if server is not connected
+    pingtime[in]   timeout in seconds for ping operation
+                   Default = 3 seconds
+    verbose[in]    if True, show ping status messages
+                   Default = False
+
+    Returns string - state
+    """
+    from mysql.utilities.common.tools import ping_host
+
+    if verbose:
+        print "# Attempting to contact %s ..." % host,
+    if server is not None and server.is_alive():
+        if verbose:
+            print "Success"
+        return "UP"
+    elif ping_host(host, pingtime):
+        if verbose:
+            print "Server is reachable"
+        return "WARN"
+    if verbose:
+        print "FAIL"
+    return "DOWN"
+
+
 def connect_servers(src_val, dest_val, options={}):
     """Connect to a source and destination server.
 
@@ -278,26 +325,26 @@ def connect_servers(src_val, dest_val, options={}):
     if isinstance(src_val, Server):
         source = src_val
     else:
-        source = _connect_server(src_name, src_dict, quiet)
+        source = get_server(src_name, src_dict, quiet)
         if not quiet:
             print "connected."
     if not _require_version(source, version):
         raise UtilError("The %s version is incompatible. Utility "
                         "requires version %s or higher." %
-                        (source.name, version))
+                        (src_name, version))
 
     # If not cloning, connect to the destination server and check version
     if not cloning:
         if isinstance(dest_val, Server):
             destination = dest_val
         else:
-            destination = _connect_server(dest_name, dest_dict, quiet)
+            destination = get_server(dest_name, dest_dict, quiet)
             if not quiet:
                 print "connected."
         if not _require_version(destination, version):
             raise UtilError("The %s version is incompatible. Utility "
                             "requires version %s or higher." %
-                            (destination.name, version))
+                            (dest_name, version))
     elif not quiet and dest_dict is not None and \
          not isinstance(dest_val, Server):
         _print_connection(dest_name, src_dict)
@@ -305,7 +352,7 @@ def connect_servers(src_val, dest_val, options={}):
     return (source, destination)
 
 
-def test_connect(conn_info):
+def test_connect(conn_info, throw_errors=False):
     """Test connection to a server.
     
     The method accepts one of the following types for conn_info:
@@ -316,7 +363,10 @@ def test_connect(conn_info):
         - an instance of the Server class
         
     conn_info[in]          Connection information
-
+    
+    throw_errors           throw any errors found during the test,
+                           false by default.
+    
     Returns True if connection success, False if error
     """
     # Parse source connection values
@@ -328,11 +378,29 @@ def test_connect(conn_info):
             'dest_name' : None,
         }
         s = connect_servers(src_val, None, conn_options)
-    except UtilError, e:
+        s[0].disconnect()
+    except UtilError:
+        if throw_errors:
+            raise
         return False
     return True
 
 
+def check_hostname_alias(server1_vals, server2_vals):
+    """Check to see if the servers are the same machine by host name.
+    
+    server1_vals[in]   connection dictionary for server1
+    server2_vals[in]   connection dictionary for server2
+    
+    Returns bool - true = server1 and server2 are the same host
+    """
+    server1 = Server({'conn_info' : server1_vals})
+    server2 = Server({'conn_info' : server2_vals})
+
+    return (server1.is_alias(server2.host) and
+            int(server1.port) == int(server2.port))
+
+
 class Server(object):
     """The Server class can be used to connect to a running MySQL server.
     The following utilities are provided:
@@ -366,18 +434,19 @@ class Server(object):
             charset        Default character set for the connection.
                            (default latin1)
         """
-        
         assert not options.get("conn_info") == None
         
         self.verbose = options.get("verbose", False)
         self.db_conn = None
+        self.host = None
         self.charset = options.get("charset", "latin1")
         self.role = options.get("role", "Server")
         conn_values = get_connection_dictionary(options.get("conn_info"))
         try:
             self.host = conn_values["host"]
             self.user = conn_values["user"]
-            self.passwd = conn_values["passwd"]
+            self.passwd = conn_values["passwd"] \
+                          if "passwd" in conn_values else None
             self.socket = conn_values["unix_socket"] \
                           if "unix_socket" in conn_values else None
             self.port = 3306
@@ -388,8 +457,109 @@ class Server(object):
         self.connect_error = None
         # Set to TRUE when foreign key checks are ON. Check with
         # foreign_key_checks_enabled.
-        self.fkeys = None  
+        self.fkeys = None
+        self.read_only = False
+        self.aliases = []
+        self.is_alias("")
+        
 
+    def is_alive(self):
+        """Determine if connection to server is still alive.
+        
+        Returns bool - True = alive, False = error or cannot connect.
+        """
+        res = True
+        try:
+            if self.db_conn is None:
+                res = False
+            else:
+                # ping and is_connected only work partially, try exec_query
+                # to make sure connection is really alive
+                retval = self.db_conn.is_connected()
+                if retval:
+                    self.exec_query("SHOW DATABASES")
+                else:
+                    res = False
+        except:
+            res = False
+        return res
+
+
+    def is_alias(self, host_or_ip):
+        """Determine if host_or_ip is an alias for this host
+        
+        host_or_ip[in] host or IP number to check
+        
+        Returns bool - True = host_or_ip is an alias
+        """
+        from mysql.utilities.common.options import hostname_is_ip
+        import socket
+        
+        if self.aliases:
+            return host_or_ip.lower() in self.aliases
+
+        # First, get the local information
+        try:
+            local_info = socket.gethostbyname_ex(socket.gethostname())
+            local_aliases = [local_info[0].lower()]
+            # if dotted host name, take first part and use as an alias
+            try:
+                local_aliases.append(local_info[0].split('.')[0])
+            except:
+                pass
+            local_aliases.extend(['127.0.0.1', 'localhost'])
+            local_aliases.extend(local_info[1])
+            local_aliases.extend(local_info[2])
+        except (socket.herror, socket.gaierror):
+            local_aliases = []
+        
+        # Check for local
+        if self.host in local_aliases:
+            self.aliases.extend(local_aliases)
+        else:
+            self.aliases.append(self.host)
+            if hostname_is_ip(self.host): # IP address
+                try:
+                    my_host = socket.gethostbyaddr(self.host)
+                    self.aliases.append(my_host[0])
+                    host_ip = socket.gethostbyname_ex(my_host[0])
+                except Exception, e:
+                    host_ip = ([],[],[])
+                    if self.verbose:
+                        print "WARNING: IP lookup failed", e
+            else:
+                try:
+                    host_ip = socket.gethostbyname_ex(self.host)
+                    self.aliases.append(host_ip[0])
+                except Exception, e:
+                    host_ip = ([],[],[])
+                    if self.verbose:
+                        print "WARNING: Hostname lookup failed", e
+
+            self.aliases.extend(host_ip[1])
+            self.aliases.extend(host_ip[2])
+
+        return host_or_ip.lower() in self.aliases
+
+
+    def user_host_exists(self, user, host_or_ip):
+        """Check to see if a user, host exists
+        
+        This method attempts to see if a user name matches the users on the
+        server and that any user, host pair can match the host or IP address
+        specified. This attempts to resolve wildcard matches.
+        
+        user[in]       user name
+        host_or_ip[in] host or IP address
+        
+        Returns string - host from server that matches the host_or_ip or
+                         None if no match.
+        """
+        res = self.exec_query("SELECT host FROM mysql.user WHERE user = '%s' AND '%s' LIKE host " % (user, host_or_ip))
+        if res:
+            return res[0][0]
+        return None
+        
 
     def get_connection_values(self):
         """Return a dictionary of connection values for the server.
@@ -434,10 +604,39 @@ class Server(object):
             parameters['charset'] = self.charset
             self.db_conn = mysql.connector.connect(**parameters)
         except mysql.connector.Error, e:
-            raise UtilDBError("Cannot connect to the %s server.\n"
-                              "Error %s" % (self.role, e.msg), e.errno)
-            return False
+            # Reset any previous value if the connection cannot be established,
+            # before raising an exception. This prevents the use of a broken
+            # database connection.
+            self.db_conn = None
+            raise UtilError("Cannot connect to the %s server.\n"
+                            "Error %s" % (self.role, e.msg), e.errno)
         self.connect_error = None
+        self.read_only = self.show_server_variable("READ_ONLY")[0][1]
+        
+
+    def disconnect(self):
+        """Disconnect from the server.
+        """
+        try:
+            self.db_conn.disconnect()
+        except:
+            pass
+
+
+    def get_version(self):
+        """Return version number of the server.
+
+        Returns string - version string or None if error
+        """
+        version_str = None
+        try:
+            res = self.show_server_variable("VERSION")
+            if res:
+                version_str = res[0][1]
+        except:
+            pass
+            
+        return version_str
 
 
     def check_version_compat(self, t_major, t_minor, t_rel):
@@ -452,25 +651,15 @@ class Server(object):
         Returns bool True if server version is GE (>=) version specified,
                      False if server version is LT (<) version specified
         """
-        res = self.show_server_variable("VERSION")
-        if res:
-            version_str = res[0][1]
-            index = version_str.find("-")
-            if index >= 0:
-                parts = res[0][1][0:index].split(".")
+        version_str = self.get_version()
+        if version_str is not None:
+            match = re.match(r'^(\d+\.\d+(\.\d+)*).*$', version_str.strip())
+            if match:
+                version = [int(x) for x in match.group(1).split('.')]
+                version = (version + [0])[:3]  # Ensure a 3 elements list
+                return version >= [int(t_major), int(t_minor), int(t_rel)]
             else:
-                parts = res[0][1].split(".")
-            major = parts[0]
-            minor = parts[1]
-            rel = parts[2]
-            if int(t_major) > int(major):
                 return False
-            elif int(t_major) == int(major):
-                if int(t_minor) > int(minor):
-                    return False
-                elif int(t_minor) == int(minor):
-                    if int(t_rel) > int(rel):
-                        return False
         return True
 
 
@@ -513,7 +702,7 @@ class Server(object):
                 cur = self.db_conn.cursor(
                     cursor_class=mysql.connector.cursor.MySQLCursorBufferedRaw)
             else:
-                cur = self.db_conn.cursor(buffered)
+                cur = self.db_conn.cursor(buffered=True)
         else:
             cur = self.db_conn.cursor(raw=True)
 
@@ -526,8 +715,8 @@ class Server(object):
             cur.close()
             raise UtilDBError("Query failed. " + e.__str__())
         except Exception, e:
-            cursor.close()
-            raise UtilError("Unknown error. Command: %s" % query_str, e.errno)
+            cur.close()
+            raise UtilError("Unknown error. Command: %s" % query_str)
         if fetch or columns:
             try:
                 results = cur.fetchall()
@@ -561,6 +750,134 @@ class Server(object):
         return self.exec_query("SHOW VARIABLES LIKE '%s'" % variable)
 
 
+    def get_uuid(self):
+        """Return the uuid for this server if it is GTID aware.
+        
+        Returns uuid or None if server is not GTID aware.
+        """
+        if self.supports_gtid() != "NO":
+            res = self.show_server_variable("server_uuid")
+            return res[0][1]
+        return None
+    
+
+    def supports_gtid(self):
+        """Determine if server supports GTIDs
+        
+        Returns string - 'ON' = gtid supported and turned on,
+                         'OFF' = supported but not enabled,
+                         'NO' = not supported
+        """
+        # Check servers for GTID support
+        version_ok = self.check_version_compat(5,6,5)
+        if not version_ok:
+            return "NO"
+        try:
+            res = self.exec_query("SELECT @@GLOBAL.GTID_MODE")
+        except:
+            return "NO"
+
+        return res[0][0]
+        
+        
+    def check_gtid_version(self):
+        """Determine if server supports latest GTID changes
+        
+        This method checks the server to ensure it contains the latest
+        changes to the GTID variables (from version 5.6.9).
+        
+        Raises UtilRplError when errors occur.
+        """
+        errors = []
+        if not self.supports_gtid() == "ON":
+            errors.append("    GTID is not enabled.")
+        if not self.check_version_compat(5, 6, 9):
+            errors.append("    Server version must be 5.6.9 or greater.")
+        res = self.exec_query("SHOW VARIABLES LIKE 'gtid_executed'")
+        if res == [] or not res[0][0] == "gtid_executed":
+            errors.append("    Missing gtid_executed system variable.")
+        if errors:
+            errors = "\n".join(errors)
+            errors = "\n".join([_GTID_ERROR % (self.host, self.port), errors])
+            raise UtilRplError(errors)
+            
+            
+    def check_gtid_executed(self, operation="copy"):
+        """Check to see if the gtid_executed variable is clear
+        
+        If the value is not clear, raise an error with appropriate instructions
+        for the user to correct the issue.
+        
+        operation[in]  Name of the operation (copy, import, etc.)
+                       default = copy
+        """
+        res = self.exec_query("SHOW GLOBAL VARIABLES LIKE 'gtid_executed'")[0]
+        if res[1].strip() == '':
+            return
+        err = ("The {0} operation contains GTID statements "
+               "that require the global gtid_executed system variable on the "
+               "target to be empty (no value). The gtid_executed value must "
+               "be reset by issuing a RESET MASTER command on the target "
+               "prior to attempting the {0} operation. "
+               "Once the global gtid_executed value is cleared, you may "
+               "retry the {0}.").format(operation)
+        raise UtilRplError(err)
+
+
+    def get_gtid_status(self):
+        """Get the GTID information for the server.
+        
+        This method attempts to retrieve the GTID lists. If the server
+        does not have GTID turned on or does not support GTID, the method
+        will throw and exception.
+        
+        Returns [list, list, list]
+        """
+        # Check servers for GTID support
+        if not self.supports_gtid():
+            raise UtilError("Global Transaction IDs are not supported.")
+            
+        res = self.exec_query("SELECT @@GLOBAL.GTID_MODE")
+        if res[0][0].upper() == 'OFF':
+            raise UtilError("Global Transaction IDs are not enabled.")
+
+        gtid_data = [self.exec_query("SELECT @@GLOBAL.GTID_EXECUTED")[0],
+                     self.exec_query("SELECT @@GLOBAL.GTID_PURGED")[0],
+                     self.exec_query("SELECT @@GLOBAL.GTID_OWNED")[0]]
+            
+        return gtid_data
+
+
+    def check_rpl_user(self, user, host):
+        """Check replication user exists and has the correct privileges.
+        
+        user[in]      user name of rpl_user
+        host[in]      host name of rpl_user
+
+        Returns [] - no exceptions, list if exceptions found
+        """
+        
+        from mysql.utilities.common.user import User
+        
+        errors = []
+        if host == '127.0.0.1':
+            host = 'localhost'
+        result = self.user_host_exists(user, host)
+        if result is None or result == []:
+            errors.append("The replication user %s@%s was not found "
+                          "on %s:%s." % (user, host, self.host, self.port))
+        else:
+            rpl_user = User(self, "%s@" % user + result)
+            if not rpl_user.has_privilege('*', '*',
+                                          'REPLICATION SLAVE'):
+                errors.append("Replication user does not have the "
+                              "correct privilege. She needs "
+                              "'REPLICATION SLAVE' on all replicated "
+                              "databases.")
+
+        return errors
+
+    
     def get_all_databases(self):
         """Return a result set containing all databases on the server
         except for internal databases (mysql, INFORMATION_SCHEMA,
@@ -824,9 +1141,9 @@ class Server(object):
         
         Returns bool - True - binary logging is ON, False = OFF
         """
-        res = self.show_server_variable("sql_log_bin")
+        res = self.show_server_variable("log_bin")
         if not res:
-            raise UtilDBError("Cannot retrieve status of sql_log_bin variable.")
+            raise UtilRplError("Cannot retrieve status of log_bin variable.")
         if res[0][1] in ("OFF", "0"):
             return False
         return True
@@ -876,3 +1193,69 @@ class Server(object):
             res = self.exec_query(_FOREIGN_KEY_SET % value, {'fetch':'false'})
 
 
+    def get_server_id(self):
+        """Retrieve the server id.
+        
+        Returns int - server id.
+        """
+        try:
+            res = self.show_server_variable("server_id")
+        except:
+            raise UtilRplError("Cannot retrieve server id from "
+                               "%s." % self.role)
+        
+        return int(res[0][1])
+
+
+    def get_server_uuid(self):
+        """Retrieve the server uuid.
+        
+        Returns string - server uuid.
+        """
+        try:
+            res = self.show_server_variable("server_uuid")
+            if res is None or res == []:
+                return None
+        except:
+            raise UtilRplError("Cannot retrieve server_uuid from "
+                               "%s." % self.role)
+        
+        return res[0][1]
+
+
+    def get_lctn(self):
+        """Get lower_case_table_name setting.
+        
+        Returns lctn value or None if cannot get value
+        """
+        res = self.show_server_variable("lower_case_table_names")
+        if res != []:
+            return res[0][1]
+        return None
+
+    
+    def get_binary_logs(self, options={}):
+        """Return a list of the binary logs.
+        
+        options[in]        query options
+        
+        Returns list - binlogs or None if binary logging turned off
+        """
+        if self.binlog_enabled():
+            return self.exec_query("SHOW BINARY LOGS", options)
+            
+        return None
+    
+
+    def set_read_only(self, on=False):
+        """Turn read only mode on/off
+        
+        on[in]         if True, turn read_only ON
+                       Default is False
+        """
+        # Only turn on|off read only if it were off at connect()
+        if not self.read_only:
+            return self.exec_query("SET @@GLOBAL.READ_ONLY = %s" %
+                                   "ON" if on else "OFF")
+        return None
+
