@@ -20,8 +20,6 @@ This module contains abstractions of MySQL replication functionality.
 """
 
 import logging
-import os
-import re
 import time
 from mysql.utilities.common.lock import Lock
 from mysql.utilities.common.replication import Master, Slave, Replication
@@ -36,8 +34,8 @@ _HEALTH_DETAIL_COLS = ["version", "master_log_file", "master_log_pos",
                        "Remaining_Delay", "IO_Error_Num", "IO_Error",
                        "SQL_Error_Num", "SQL_Error", "Trans_Behind"]
 
-_GTID_DONE = "SELECT @@GLOBAL.GTID_DONE"
-_GTID_WAIT = "SELECT SQL_THREAD_WAIT_AFTER_GTIDS('%s', %s)"
+_GTID_EXECUTED = "SELECT @@GLOBAL.GTID_EXECUTED"
+_GTID_WAIT = "SELECT WAIT_UNTIL_SQL_THREAD_AFTER_GTIDS('%s', %s)"
 
 
 def parse_failover_connections(options):
@@ -147,8 +145,9 @@ class Topology(Replication):
         self.force = self.options.get("force", False)
         self.before_script = self.options.get("before", None)
         self.after_script = self.options.get("after", None)
-        self.timeout = self.options.get("timeout", 3)
+        self.timeout = int(self.options.get("timeout", 300))
         self.logging = self.options.get("logging", False)
+        self.rpl_user = self.options.get("rpl_user", None)
         
         # Attempt to connect to all servers        
         self.master, self.slaves = self._connect_to_servers(master_vals,
@@ -283,6 +282,23 @@ class Topology(Replication):
         self.slaves = new_list
 
 
+    def check_master_info_type(self, repo="TABLE"):
+        """Check all slaves for master_info_repository=repo
+        
+        repo[in]       value for master info = "TABLE" or "FILE"
+                       Default is "TABLE"
+
+        Returns bool - True if master_info_repository == repo
+        """
+        for slave_dict in self.slaves:
+            slave = slave_dict['instance']
+            if slave is not None:
+                res = slave.show_server_variable("master_info_repository")
+                if not res or res[0][1].upper() != repo.upper():
+                    return False
+        return True
+
+
     def discover_slaves(self, skip_conn_err=True):
         """Discover slaves connected to the master
         
@@ -298,17 +314,22 @@ class Topology(Replication):
         # Get user and password
         try:
             user, password = discover.split(":")
-        except:
+        except ValueError:
             user = discover
-            passord = None
+            password = None
             
         # Find discovered slaves
         new_slaves_found = False
         self._report("# Discovering slaves for master at %s:%s" %
-                     (self.master.host, self.master.port), False)
-        discovered_slaves = self.master.get_slaves()
+                     (self.master.host, self.master.port))
+        discovered_slaves = self.master.get_slaves(user, password)
         for slave in discovered_slaves:
             host, port = slave.split(":")
+            self._report("Discovering slave at %s:%s" % (host, port),
+                         logging.INFO, False)
+            # Convert local IP to localhost
+            if host == '127.0.0.1':
+                host = 'localhost'
             # Skip hosts that are not registered properly
             if host == 'unknown host':
                 continue
@@ -320,6 +341,7 @@ class Topology(Replication):
                     if slave_dict['host'] == host and \
                        int(slave_dict['port']) == int(port):
                         found = True
+                        break
                 if not found:
                     # Now we must attempt to connect to the slave.
                     conn_dict = {
@@ -332,22 +354,22 @@ class Topology(Replication):
                     slave_conn = Slave(conn_dict)
                     try:
                         slave_conn.connect()
-                        m_host = self.master.host
-                        m_port = self.master.port
                         # Skip discovered slaves that are not connected
                         # to the master
-                        if slave_conn.is_connected_to_master(m_host, m_port) \
-                           and slave_conn.is_connected():
+                        if slave_conn.is_connected():
                             self.slaves.append({ 'host' : host, 'port' : port,
                                                  'instance' : slave_conn,
                                                  'discovered' : True})
-                            new_slaves_found = True                            
-                    except:
-                        msg = "Cannot connect to slave %s:%s as user '%s'." % \
+                            self._report("Found slave: %s:%s" %
+                                         (host, port), logging.INFO, False)
+                            new_slaves_found = True
+                        else:
+                            self._report("Not found.", logging.WARN, False)
+                    except UtilError, e:
+                        msg = "Cannot connect to slave %s:%s as user '%s'. " % \
                               (host, port, user)
                         if skip_conn_err:
-                            if self.verbose:
-                                self._report("# ERROR: %s" % msg, logging.ERROR)
+                            self._report(msg + e.errmsg, logging.WARN, False)
                         else:
                             raise UtilRplError(msg)
         
@@ -448,8 +470,8 @@ class Topology(Replication):
             # Create replication user if --force is specified.
             if self.force and candidate_ok[1] == "RPL_USER":
                 user, passwd = slave.get_rpl_user()
-                m_candidate.create_rpl_user("%s:%s" % (user, passwd),
-                                            slave.host, slave.port)
+                candidate.create_rpl_user(slave.host, slave.port,
+                                          user, passwd)
             else:
                 msg = candidate_ok[2]
                 self._report(msg, logging.CRITICAL)
@@ -466,10 +488,20 @@ class Topology(Replication):
         
         Returns tuple - user, password
         """
-        slave = self._change_role(server)
-        user, passwd = slave.get_rpl_user()
-        return user, passwd
+        # Get replication user from server if rpl_user not specified
+        if self.rpl_user is None:
+            slave = self._change_role(server)
+            user, passwd = slave.get_rpl_user()
+            return (user, passwd)
 
+        # Get user and password
+        try:
+            user, passwd =  self.rpl_user.split(":")
+        except ValueError:
+            user = self.rpl_user.strip(":")
+            passwd = None
+        return (user, passwd)
+    
     
     def run_script(self, script, quiet):
         """Run an external script
@@ -583,8 +615,7 @@ class Topology(Replication):
                     self._report(msg % "FAIL", logging.WARN)
                 return (False, "CONNECTED",
                         "Connection to slave server lost.")            
-            if not slave.is_connected_to_master(self.master.host,
-                                                self.master.port):
+            if not slave.is_configured_for_master(self.master):
                 if self.verbose and not quiet:
                     self._report(msg % "FAIL", logging.WARN)
                 return (False, "CONNECTED",
@@ -628,13 +659,17 @@ class Topology(Replication):
                 
         # Check replication user - must exist with correct privileges
         user, passwd = slave.get_rpl_user()
-        if user is None or slave.check_rpl_user(user, passwd) == []:
-            msg = "#   Replication user exists ... %s"
-            if self.verbose and not force and not quiet:
-                self._report(msg % "FAIL", logging.WARN)
-            return (False, "RPL_USER",
-                    "Candidate slave is missing replication user.")
-        if self.verbose and not self.force and not quiet:
+        msg = "#   Replication user exists ... %s"
+        if user is None or slave.check_rpl_user(user, slave.host) != []:
+            if not self.force:
+                if self.verbose and not quiet:
+                    self._report(msg % "FAIL", logging.WARN)
+                return (False, "RPL_USER",
+                        "Candidate slave is missing replication user.")
+            else:
+                self._report("Replication user not found but --force used.",
+                             logging.WARN)
+        elif self.verbose and not quiet:
             self._report(msg % "Ok")
                 
         # If no GTIDs, we need binary logging enabled on candidate.
@@ -700,7 +735,7 @@ class Topology(Replication):
                 continue
 
             # Sanity check: ensure candidate and slave are not the same.
-            if s_host == candidate.host and int(s_port) == int(candidate.port):
+            if candidate.is_alias(s_host) and int(s_port) == int(candidate.port):
                 continue
             
             res = candidate.stop()
@@ -730,6 +765,8 @@ class Topology(Replication):
             lock_ftwrl.unlock()
 
             res = candidate.start()
+            candidate.exec_query("COMMIT")
+    
             if res is None or res != () and not self.quiet:
                 self._report("Candidate %s:%s failed to start." % 
                              (hostport, res[0]))
@@ -737,7 +774,7 @@ class Topology(Replication):
             if self.verbose and not self.quiet:
                 self._report("# Waiting for candidate to catch up to slave " 
                              "%s:%s." % (s_host, s_port))
-            master_gtid = master.exec_query(_GTID_DONE)
+            master_gtid = master.exec_query(_GTID_EXECUTED)
             candidate.wait_for_slave_gtid(master_gtid, self.timeout,
                                           self.verbose and not self.quiet)
             
@@ -870,6 +907,8 @@ class Topology(Replication):
         """    
         assert (self.master is not None), "No master or connection failed."
 
+        import operator
+        
         # Get master health
         rpl_health = self.master.check_rpl_health()
         self._report("# Getting health for master: %s:%s." %
@@ -903,19 +942,33 @@ class Topology(Replication):
     
         rows.append(master_data)
         
+        slave_rows = []
         # Get the health of the slaves
         if have_gtid == "ON":
-            master_gtids = self.master.exec_query(_GTID_DONE)
+            master_gtids = self.master.exec_query(_GTID_EXECUTED)
         for slave_dict in self.slaves:
             host = slave_dict['host']
             port = slave_dict['port']
             slave = slave_dict['instance']
-            if slave is None or \
-               (slave is not None and (not slave.is_alive() or \
-               not slave.is_connected_to_master(self.master.host,
-                                                self.master.port))):
+            if slave is None:
                 rpl_health = (False, ["Cannot connect to slave."])
+            elif not slave.is_alive():
+                # Attempt to reconnect to the database server.
+                try:
+                    slave.connect()
+                    # Connection succeeded.
+                    if not slave.is_configured_for_master(self.master):
+                        rpl_health = (False,
+                                      ["Slave is not connected to master."])
+                        slave = None
+                except UtilError:
+                    # Connection failed.
+                    rpl_health = (False, ["Slave is not alive."])
+                    slave = None
+            elif not slave.is_configured_for_master(self.master):
+                rpl_health = (False, ["Slave is not connected to master."])
                 slave = None
+
             if slave is not None:
                 rpl_health = slave.check_rpl_health(self.master,
                                                     master_log, master_log_pos,
@@ -942,7 +995,7 @@ class Topology(Replication):
             # Show additional details if verbosity turned on
             if self.verbosity > 0:
                 if slave is None:
-                    slave_data.extend(["","","","","","","","","","","","",""])
+                    slave_data.extend([""]*13)
                 else:
                     slave_data.append(slave.get_version())
                     res = slave.get_rpl_details()
@@ -954,10 +1007,14 @@ class Topology(Replication):
                         else:
                             slave_data.extend([""])
                     else:
-                        slave_data.extend(["","","","","","","","","","","","",""])
+                        slave_data.extend([""]*13)
                                    
-            rows.append(slave_data)
-    
+            slave_rows.append(slave_data)
+
+        # order the slaves
+        slave_rows.sort(key=operator.itemgetter(0, 1))
+        rows.extend(slave_rows)
+
         return (columns, rows)
         
         
@@ -1026,7 +1083,10 @@ class Topology(Replication):
             servers.append(self.master)
             for slave_conn in self.slaves:
                 slave = slave_conn['instance']
-                servers.append(slave)
+                # A slave instance is None if the connection failed during the
+                # creation of the topology. In this case ignore the slave.
+                if slave is not None:
+                    servers.append(slave)
 
         # If candidates were specified, check those too.
         candidates = self.options.get("candidates", None)
@@ -1077,7 +1137,7 @@ class Topology(Replication):
         
         self._report("# Performing %s on all slaves." %
                      command.upper())
-        i = 0
+
         for slave_dict in self.slaves:
             hostport = "%s:%s" % (slave_dict['host'], slave_dict['port'])
             msg = "#   Executing %s on slave %s " % (command, hostport)
@@ -1107,7 +1167,6 @@ class Topology(Replication):
                                  logging.WARN)
                 elif not quiet:
                     self._report(msg + "Ok")
-            i += 1
             
             
     def connect_candidate(self, candidate, master=True):
@@ -1155,7 +1214,6 @@ class Topology(Replication):
         
         Return bool - True = success, raises exception on error
         """
-        from mysql.utilities.common.server import get_connection_dictionary
 
         # Need instance of Master class for operation
         m_candidate = self.connect_candidate(candidate)
@@ -1172,14 +1230,10 @@ class Topology(Replication):
 
         user, passwd = self._get_rpl_user(m_candidate)
         # Create user on m_candidate.
-        if passwd:
-            rpl_user = "%s:%s" % (user, passwd)
-        else:
-            rpl_user = user
         if self.verbose:
             self._report("# Creating replication user if it does not exist.")
-        res = m_candidate.create_rpl_user(rpl_user, m_candidate.host,
-                                           m_candidate.port)
+        res = m_candidate.create_rpl_user(m_candidate.host, m_candidate.port,
+                                          user, passwd)
 
         # Call exec_before script - display output if verbose on
         self.run_script(self.before_script, False)
@@ -1198,7 +1252,7 @@ class Topology(Replication):
         # Wait for all slaves to catch up.
         gtid_enabled = self.master.supports_gtid() == "ON"
         if gtid_enabled:
-            master_gtid = self.master.exec_query(_GTID_DONE)
+            master_gtid = self.master.exec_query(_GTID_EXECUTED)
         self._report("# Waiting for slaves to catch up to old master.")
         for slave_dict in self.slaves:
             master_info = self.master.get_status()[0]
@@ -1354,6 +1408,9 @@ class Topology(Replication):
                                                          slave, check_master)
             if slave_ok is not None and slave_ok[0]:
                 return slave_dict
+            else:
+                self._report("# Candidate %s:%s does not meet the requirements." %
+                             (slave.host, slave.port), logging.WARN)
             
         # If strict is on and we have found no viable candidates, return None
         if strict:
@@ -1377,7 +1434,9 @@ class Topology(Replication):
                                                              check_master)
                 if slave_ok is not None and slave_ok[0]:
                     return slave_dict
-            except:
+            except UtilError, e:
+                self._report("# Slave eliminated due to error: %s" % e.errmsg,
+                             logging.WARN)
                 pass # Slave gone away, skip it.
             
         return None
@@ -1445,6 +1504,17 @@ class Topology(Replication):
                 self._report(msg, logging.CRITICAL)
                 raise UtilRplError(msg)
 
+        # We must also ensure the new master and all remaining slaves
+        # have the latest GTID support.
+        new_master.check_gtid_version()
+        for slave_dict in self.slaves:
+            # Ignore dead or offline slaves
+            slave = slave_dict['instance']
+            # skip dead or zombie slaves
+            if slave is None or not slave.is_alive():
+                continue
+            slave.check_gtid_version()
+
         host = new_master_dict['host']
         port = new_master_dict['port']
         self._report("# Candidate slave %s:%s will become the new master." % 
@@ -1457,15 +1527,11 @@ class Topology(Replication):
         self._prepare_candidate_for_failover(new_master, user, passwd)
 
         # Create replication user on candidate.
-        if passwd:
-            rpl_user = "%s:%s" % (user, passwd)
-        else:
-            rpl_user = user
         self._report("# Creating replication user if it does not exist.")
             
         # Need Master class instance to check master and replication user
         self.master = self._change_role(new_master, False)
-        res = self.master.create_rpl_user(rpl_user, host, port)
+        res = self.master.create_rpl_user(host, port, user, passwd)
 
         # Call exec_before script - display output if verbose on
         self.run_script(self.before_script, False)
