@@ -30,13 +30,14 @@ from mysql.utilities.common.options import setup_common_options
 from mysql.utilities.common.options import parse_connection, add_rpl_user
 from mysql.utilities.common.options import add_verbosity, check_verbosity
 from mysql.utilities.command.setup_rpl import setup_replication
+from mysql.utilities.common.server import check_hostname_alias
 from mysql.utilities import VERSION_FRM
 
 # Constants
 NAME = "MySQL Utilities - mysqlreplicate "
 DESCRIPTION = "mysqlreplicate - establish replication with a master"
 USAGE = "%prog --master=root@localhost:3306 --slave=root@localhost:3310 " \
-        "--server-id=3 --rpl_user=rpl:passwd "
+        "--rpl-user=rpl:passwd "
 
 # Setup the command parser
 parser = setup_common_options(os.path.basename(sys.argv[0]),
@@ -104,6 +105,10 @@ try:
 except:
     parser.error("Slave connection values invalid or cannot be parsed.")
     
+# Check hostname alias
+if check_hostname_alias(m_values, s_values):
+    parser.error("The master and slave are the same host and port.")
+    
 # Check required --master-log-file for --master-log-pos
 if (opt.master_log_pos >= 0 and opt.master_log_file is None):
     parser.error("You must specify a master log file to use the master "
